39#define CLOCK_BITSIZE_MIN 0.2f
40#define CLOCK_BITSIZE_MAX 1.5f
41#define SYNC_BITSIZE_MIN 12.f
42#define SYNC_BITSIZE_MAX 15.f
88#define OFFSET(x) offsetof(ReadEIA608Context, x)
89#define FLAGS AV_OPT_FLAG_VIDEO_PARAM|AV_OPT_FLAG_FILTERING_PARAM|AV_OPT_FLAG_RUNTIME_PARAM
132 if (end >= inlink->
h) {
142 if (
s->nb_allocated < end - start + 1) {
143 const int diff = end - start + 1 -
s->nb_allocated;
148 memset(&
s->scan[
s->nb_allocated], 0,
diff *
sizeof(*
s->scan));
149 s->nb_allocated = end - start + 1;
152 for (
int i = 0;
i <
s->nb_allocated;
i++) {
181 const int max =
s->max;
182 int start = 0, end = 0, middle;
183 int black = 0, white = 0;
186 for (
int i = 0;
i <=
max;
i++) {
193 for (
int i =
max;
i >= 0;
i--) {
200 middle = start + (end - start) / 2;
203 for (
int i = start;
i <= middle;
i++) {
211 for (
int i = end;
i >= middle;
i--) {
224 float sum = 0.0,
mean = 0.0;
226 for (
int i = 0;
i <
len;
i++)
227 sum +=
line[
i].filtered;
237 float standard_deviation = 0.f;
239 for (
int i = 0;
i <
len;
i++)
240 standard_deviation += (
line[
i].filtered - m) * (
line[
i].filtered - m);
242 return sqrtf(standard_deviation / (
len - 1));
246 int lag,
float threshold,
float influence,
int len)
248 for (
int i = lag;
i <
len + lag;
i++) {
253 for (
int i = 0;
i < lag;
i++) {
261 for (
int i = lag;
i <
len + lag;
i++) {
264 line[
i].output = 255;
269 line[
i].filtered = influence *
line[
i].unfiltered + (1.f - influence) *
line[
i-1].filtered;
271 int distance_from_black, distance_from_white;
276 line[
i].output = distance_from_black <= distance_from_white ? 0 : 255;
286 int hold =
line[
LAG].output, cnt = 0;
292 if (
line[
i].output != hold) {
293 code[cnt].size =
i - last;
294 code[cnt].bit = hold;
295 hold =
line[
i].output;
302 code[cnt].bit = hold;
310 for (
int i = 0;
i <
len;
i++) {
316#define READ_LINE(type, name) \
317static void read_##name(AVFrame *in, int nb_line, LineItem *line, int lp, int w) \
319 const type *src = (const type *)(&in->data[0][nb_line * in->linesize[0]]);\
322 for (int i = 0; i < w; i++) { \
323 int a = FFMAX(i - 3, 0); \
324 int b = FFMAX(i - 2, 0); \
325 int c = FFMAX(i - 1, 0); \
326 int d = FFMIN(i + 3, w-1); \
327 int e = FFMIN(i + 2, w-1); \
328 int f = FFMIN(i + 1, w-1); \
330 line[LAG + i].input = (src[a] + src[b] + src[c] + src[i] + \
331 src[d] + src[e] + src[f] + 6) / 7; \
334 for (int i = 0; i < w; i++) { \
335 line[LAG + i].input = src[i]; \
351 s->depth =
desc->comp[0].depth;
352 s->max = (1 <<
desc->comp[0].depth) - 1;
353 s->read_line[0] = read_byte;
354 s->read_line[1] = read_word;
364 uint8_t codes[19] = { 0 };
365 float bit_size = 0.f;
372 s->read_line[
s->depth > 8](in, nb_line,
line,
s->lp,
w);
389 for (
i = 14;
i <
len;
i++) {
394 for (
i = 1;
i < 14;
i++) {
401 if (scan->
code[15].
size / bit_size < 0.45f) {
405 for (j = 0,
i = 14;
i <
len;
i++) {
411 for (
int k = 0; j < 19 && k <
run; k++) {
419 for (ch = 0; ch < 2; ch++) {
421 int b = codes[3 + ch * 8 +
i];
434 scan->
byte[ch] = 0x7F;
444 int job,
int nb_jobs)
448 const int h =
s->end -
s->start + 1;
449 const int start = (
h * job) / nb_jobs;
450 const int end = (
h * (job+1)) / nb_jobs;
453 for (
int i = start;
i < end;
i++) {
473 for (
int i = 0;
i <
s->end -
s->start + 1;
i++) {
501 for (
int i = 0;
i <
s->nb_allocated;
i++) {
514 char *res,
int res_len,
int flags)
517 int ret, start =
s->start, end =
s->end;
542 .p.name =
"readeia608",
543 .p.description =
NULL_IF_CONFIG_SMALL(
"Read EIA-608 Closed Caption codes from input video and write them to frame metadata."),
544 .p.priv_class = &readeia608_class,
static int config_input(AVFilterLink *inlink)
static int process_command(AVFilterContext *ctx, const char *cmd, const char *args, char *res, int res_len, int flags)
const FFFilter ff_vf_readeia608
int ff_filter_frame(AVFilterLink *link, AVFrame *frame)
Send a frame of data to the next filter.
int ff_filter_process_command(AVFilterContext *ctx, const char *cmd, const char *arg, char *res, int res_len, int flags)
Generic processing of user supplied commands that are set in the same way as the filter options.
int ff_filter_execute(AVFilterContext *ctx, avfilter_action_func *func, void *arg, int *ret, int nb_jobs)
int ff_filter_get_nb_threads(AVFilterContext *ctx)
Get number of threads for current filter instance.
Main libavfilter public API header.
#define flags(name, subs,...)
#define i(width, name, range_min, range_max)
#define bit(string, value)
#define FFABS(a)
Absolute value, Note, INT_MIN / INT64_MIN result in undefined behavior as they are not representable ...
static __device__ float sqrtf(float a)
static __device__ float fabsf(float a)
static int filter_frame(DBEDecodeContext *s, AVFrame *frame)
@ AV_OPT_TYPE_INT
Underlying C type is int.
@ AV_OPT_TYPE_FLOAT
Underlying C type is float.
@ AV_OPT_TYPE_BOOL
Underlying C type is int.
#define AVFILTER_FLAG_SUPPORT_TIMELINE_GENERIC
Some filters support a generic "enable" expression option that can be used to enable or disable a fil...
#define AVFILTER_FLAG_SLICE_THREADS
The filter supports multithreading by splitting frames into multiple parts and processing them concur...
#define AVFILTER_FLAG_METADATA_ONLY
The filter is a "metadata" filter - it does not modify the frame data in any way.
int av_dict_set(AVDictionary **pm, const char *key, const char *value, int flags)
Set the given entry in *pm, overwriting an existing entry.
int av_dict_set_int(AVDictionary **pm, const char *key, int64_t value, int flags)
Convenience wrapper for av_dict_set() that converts the value to a string and stores it.
#define AVERROR_BUG
Internal bug, also see AVERROR_BUG2.
#define AV_LOG_DEBUG
Stuff which is only useful for libav* developers.
#define AV_LOG_WARNING
Something somehow does not look correct.
#define AV_LOG_ERROR
Something went wrong and cannot losslessly be recovered.
static av_cold void uninit(AVBitStreamFilterContext *ctx)
#define FILTER_INPUTS(array)
#define FILTER_OUTPUTS(array)
#define FILTER_PIXFMTS_ARRAY(array)
#define AVFILTER_DEFINE_CLASS(fname)
common internal API header
#define NULL_IF_CONFIG_SMALL(x)
Return NULL if CONFIG_SMALL is true, otherwise the argument without modification.
void * av_calloc(size_t nmemb, size_t size)
Memory handling functions.
const AVPixFmtDescriptor * av_pix_fmt_desc_get(enum AVPixelFormat pix_fmt)
#define AV_PIX_FMT_YUV420P16
#define AV_PIX_FMT_YUV444P12
#define AV_PIX_FMT_YUV444P9
#define AV_PIX_FMT_YUV420P10
#define AV_PIX_FMT_YUV440P12
#define AV_PIX_FMT_YUV422P9
#define AV_PIX_FMT_YUVA444P10
#define AV_PIX_FMT_YUVA420P16
#define AV_PIX_FMT_YUV420P12
#define AV_PIX_FMT_YUVA420P10
#define AV_PIX_FMT_YUVA422P9
#define AV_PIX_FMT_YUV422P12
#define AV_PIX_FMT_YUV422P10
#define AV_PIX_FMT_GRAY12
#define AV_PIX_FMT_YUV420P9
#define AV_PIX_FMT_YUVA420P9
#define AV_PIX_FMT_YUVA422P10
#define AV_PIX_FMT_YUV420P14
AVPixelFormat
Pixel format.
@ AV_PIX_FMT_YUV420P
planar YUV 4:2:0, 12bpp, (1 Cr & Cb sample per 2x2 Y samples)
@ AV_PIX_FMT_YUV440P
planar YUV 4:4:0 (1 Cr & Cb sample per 1x2 Y samples)
@ AV_PIX_FMT_YUV422P
planar YUV 4:2:2, 16bpp, (1 Cr & Cb sample per 2x1 Y samples)
@ AV_PIX_FMT_GRAY8
Y , 8bpp.
@ AV_PIX_FMT_YUVA420P
planar YUV 4:2:0, 20bpp, (1 Cr & Cb sample per 2x2 Y & A samples)
@ AV_PIX_FMT_YUVJ440P
planar YUV 4:4:0 full scale (JPEG), deprecated in favor of AV_PIX_FMT_YUV440P and setting color_range
@ AV_PIX_FMT_YUV410P
planar YUV 4:1:0, 9bpp, (1 Cr & Cb sample per 4x4 Y samples)
@ AV_PIX_FMT_YUV411P
planar YUV 4:1:1, 12bpp, (1 Cr & Cb sample per 4x1 Y samples)
@ AV_PIX_FMT_YUV444P
planar YUV 4:4:4, 24bpp, (1 Cr & Cb sample per 1x1 Y samples)
@ AV_PIX_FMT_YUVA444P
planar YUV 4:4:4 32bpp, (1 Cr & Cb sample per 1x1 Y & A samples)
@ AV_PIX_FMT_YUVJ411P
planar YUV 4:1:1, 12bpp, (1 Cr & Cb sample per 4x1 Y samples) full scale (JPEG), deprecated in favor ...
@ AV_PIX_FMT_YUVJ422P
planar YUV 4:2:2, 16bpp, full scale (JPEG), deprecated in favor of AV_PIX_FMT_YUV422P and setting col...
@ AV_PIX_FMT_YUVA422P
planar YUV 4:2:2 24bpp, (1 Cr & Cb sample per 2x1 Y & A samples)
@ AV_PIX_FMT_YUVJ444P
planar YUV 4:4:4, 24bpp, full scale (JPEG), deprecated in favor of AV_PIX_FMT_YUV444P and setting col...
@ AV_PIX_FMT_YUVJ420P
planar YUV 4:2:0, 12bpp, full scale (JPEG), deprecated in favor of AV_PIX_FMT_YUV420P and setting col...
#define AV_PIX_FMT_YUVA422P12
#define AV_PIX_FMT_YUV422P14
#define AV_PIX_FMT_GRAY10
#define AV_PIX_FMT_GRAY14
#define AV_PIX_FMT_YUV422P16
#define AV_PIX_FMT_YUV440P10
#define AV_PIX_FMT_GRAY16
#define AV_PIX_FMT_YUVA444P16
#define AV_PIX_FMT_YUVA422P16
#define AV_PIX_FMT_YUV444P14
#define AV_PIX_FMT_YUVA444P9
#define AV_PIX_FMT_YUVA444P12
#define AV_PIX_FMT_YUV444P16
#define AV_PIX_FMT_YUV444P10
Describe the class of an AVClass context structure.
A link between two filters.
int w
agreed upon image width
int h
agreed upon image height
AVFilterContext * dst
dest filter
A filter pad used for either input or output.
This structure describes decoded (raw) audio or video data.
AVDictionary * metadata
metadata.
Descriptor that unambiguously describes how the bits of a pixel are stored in the up to 4 data planes...
void(* read_line[2])(AVFrame *in, int nb_line, LineItem *line, int lp, int w)
#define av_realloc_f(p, o, n)
static AVFormatContext * ctx
static enum AVPixelFormat pixel_fmts[]
static float mean(const float *input, int size)
static av_always_inline int diff(const struct color_info *a, const struct color_info *b, const int trans_thresh)
#define CLOCK_BITSIZE_MIN
static const AVOption readeia608_options[]
static float meanf(const LineItem *line, int len)
#define READ_LINE(type, name)
static void build_histogram(ReadEIA608Context *s, ScanItem *scan, const LineItem *line, int len)
static void extract_line(AVFilterContext *ctx, AVFrame *in, ScanItem *scan, int w, int nb_line)
static void find_black_and_white(ReadEIA608Context *s, ScanItem *scan)
static int filter_frame(AVFilterLink *inlink, AVFrame *in)
static int periods(const LineItem *line, CodeItem *code, int len)
static int config_filter(AVFilterContext *ctx, int start, int end)
static int process_command(AVFilterContext *ctx, const char *cmd, const char *args, char *res, int res_len, int flags)
static av_cold void uninit(AVFilterContext *ctx)
static float stddevf(const LineItem *line, int len)
static void dump_code(AVFilterContext *ctx, ScanItem *scan, int len, int item)
static int extract_lines(AVFilterContext *ctx, void *arg, int job, int nb_jobs)
#define CLOCK_BITSIZE_MAX
static const AVFilterPad readeia608_inputs[]
static void thresholding(ReadEIA608Context *s, ScanItem *scan, LineItem *line, int lag, float threshold, float influence, int len)
const AVFilterPad ff_video_default_filterpad[1]
An AVFilterPad array whose only entry has name "default" and is of type AVMEDIA_TYPE_VIDEO.